﻿'Copyright (c) 2009 Grailtek

'Permission is hereby granted, free of charge, to any person
'obtaining a copy of this software and associated documentation
'files (the "Software"), to deal in the Software without
'restriction, including without limitation the rights to use,
'copy, modify, merge, publish, distribute, sublicense, and/or sell
'copies of the Software, and to permit persons to whom the
'Software is furnished to do so, subject to the following
'conditions:

'The above copyright notice and this permission notice shall be
'included in all copies or substantial portions of the Software.

'THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
'EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
'OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
'NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
'HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
'WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
'FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
'OTHER DEALINGS IN THE SOFTWARE.

Imports System.Reflection

Namespace Aspect

    '<DebuggerStepThrough()> _
    <CLSCompliant(True)> _
    Public Class Argument
        Friend Sub New(ByVal iParam As ParameterInfo, ByVal iValue As Object)
            _Param = iParam
            _Value = iValue
            ParamAttribute = iParam.GetCustomAttributes(True).OfType(Of FieldAttribute).FirstOrDefault
            If Not ParamAttribute Is Nothing Then
                ParamUIName = ParamAttribute.UIName
            End If
            TypeAttribute = _Param.ParameterType.GetCustomAttributes(True).OfType(Of ItemAttribute).FirstOrDefault
            If Not TypeAttribute Is Nothing Then
                ParamUIName = TypeAttribute.UIName
            End If
        End Sub
        Private ReadOnly _Param As ParameterInfo
        Public ReadOnly Property Param() As Reflection.ParameterInfo
            Get
                Return _Param
            End Get
        End Property
        Private _Value As Object
        Public Property Value() As Object
            Get
                Return _Value
            End Get
            Set(ByVal value As Object)
                _Value = value
                RaiseEvent ValueChanged(Me, EventArgs.Empty)
            End Set
        End Property
        Public Event ValueChanged As EventHandler
        Public ReadOnly ParamAttribute As FieldAttribute
        Public ReadOnly ParamUIName As String
        Public ReadOnly TypeAttribute As ItemAttribute
        Public Function Validate() As String
            Dim ret As String = Nothing
            If Not ParamAttribute Is Nothing Then
                Dim res As String = ParamAttribute.Validate(_Param, _Value)
                If String.IsNullOrEmpty(ret) Then ret = res
            End If
            Return ret
        End Function
        Public Overrides Function ToString() As String
            Return ParamUIName & " " & Param.ParameterType.Name
        End Function
    End Class

End Namespace