﻿'Copyright (c) 2009 Grailtek

'Permission is hereby granted, free of charge, to any person
'obtaining a copy of this software and associated documentation
'files (the "Software"), to deal in the Software without
'restriction, including without limitation the rights to use,
'copy, modify, merge, publish, distribute, sublicense, and/or sell
'copies of the Software, and to permit persons to whom the
'Software is furnished to do so, subject to the following
'conditions:

'The above copyright notice and this permission notice shall be
'included in all copies or substantial portions of the Software.

'THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
'EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
'OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
'NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
'HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
'WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
'FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
'OTHER DEALINGS IN THE SOFTWARE.

Imports System.Threading
Imports System.Reflection
Imports System.Reflection.Emit

'<DebuggerStepThrough()> _
Friend NotInheritable Class ImplementorFactory

    Private Sub New()
    End Sub

    'TODO: we can wire this up as direct IL later, and since this will be a call that could be called a lot, it will actually be worth the extra effort
    Public Shared Function GetNewOf(Of T As Class)() As T
        Dim interfaceType As Type = GetType(T)
        Dim mt As Type = GetImplementingType(Of T)()
        Return DirectCast(Activator.CreateInstance(mt), T)
    End Function

    Private Shared GetImplementingTypeMethod As MethodInfo = GetType(ImplementorFactory).GetMethod("GetImplementingType", BindingFlags.Static Or BindingFlags.Public, Nothing, Type.EmptyTypes, Nothing)
    Public Shared Function GetImplementingType(Of T As Class)() As Type
        Dim interfaceType As Type = GetType(T)
        If Not interfaceType.IsInterface Then Throw New ArgumentException(String.Format("Type {0} is not an interface.", interfaceType.Name))
        Return InterfaceImplementor(Of T).MaterializedType
    End Function

    Friend Shared Function GetImplementingType(ByVal interfaceType As Type) As Type
        Debug.Assert(Not GetImplementingTypeMethod Is Nothing, "GetImplementingTypeMethod is empty.")
        Dim m As MethodInfo = GetImplementingTypeMethod.MakeGenericMethod(New Type() {interfaceType})
        Dim o As Object = m.Invoke(Nothing, Nothing)
        Return DirectCast(o, Type)
    End Function

End Class